@@ -146,7 +146,7 @@ class Element:
             return self.get_default_label()
         return f"{self.get_symbol()}_{self._label}"
 
-    def set_label(self, label: str):
+    def set_label(self, label: str) -> "Element":
         """
         Set the label assigned to a specific instance of the element.
 
@@ -157,6 +157,7 @@ class Element:
         """
         assert type(label) is str, f"{label=}"
         self._label = label.strip()
+        return self
 
     @staticmethod
     def get_symbol() -> str:
@@ -176,8 +177,8 @@ class Element:
         Parameters
         ----------
         decimals: int = -1
-            The number of decimals used when formatting the current value and the limits for the
-            element's parameters. -1 corresponds to no values being included in the output.
+            The number of decimals used when formatting the current value and the limits for the element's parameters.
+            -1 corresponds to no values being included in the output.
 
         Returns
         -------
@@ -336,7 +337,7 @@ class Element:
         assert type(key) is str and key.strip() != ""
         return self._fixed_parameters[key]
 
-    def set_fixed(self, key: str, value: bool):
+    def set_fixed(self, key: str, value: bool) -> "Element":
         """
         Set whether or not an element parameter should have a fixed value when fitting a circuit
         to a data set.
@@ -353,6 +354,7 @@ class Element:
         assert key in self._fixed_parameters, f"{key=}"
         assert type(value) is bool, f"{value=}"
         self._fixed_parameters[key] = value
+        return self
 
     def get_lower_limit(self, key: str) -> float:
         """
@@ -372,7 +374,7 @@ class Element:
         assert type(key) is str and key.strip() != ""
         return self._lower_limits[key]
 
-    def set_lower_limit(self, key: str, value: float):
+    def set_lower_limit(self, key: str, value: float) -> "Element":
         """
         Set the upper limit for the value of an element parameter when fitting a circuit to a data
         set.
@@ -389,6 +391,7 @@ class Element:
         assert type(key) is str and key.strip() != ""
         assert key in self._lower_limits, f"{key=}"
         self._lower_limits[key] = float(value)
+        return self
 
     def get_upper_limit(self, key: str) -> float:
         """
@@ -408,7 +411,7 @@ class Element:
         assert type(key) is str and key.strip() != ""
         return self._upper_limits[key]
 
-    def set_upper_limit(self, key: str, value: float):
+    def set_upper_limit(self, key: str, value: float) -> "Element":
         """
         Set the upper limit for the value of an element parameter when fitting a circuit to a data
         set.
@@ -425,6 +428,7 @@ class Element:
         assert type(key) is str and key.strip() != ""
         assert key in self._upper_limits, f"{key=}"
         self._upper_limits[key] = float(value)
+        return self
 
     def _str_expr(self, substitute: bool = False) -> str:
         raise Exception("Method has not been implemented!")
@@ -442,11 +446,10 @@ class Element:
             if symbols_only:
                 if self._label != "":
                     repl = f"{k}_{self._label}"
-                else:
-                    assert (
-                        self._identifier >= 0
-                    ), "Assign an identifier, set a label, or create the element by parsing a circuit description code!"
+                elif self._identifier >= 0:
                     repl = f"{k}_{self._identifier}"
+                else:
+                    repl = f"{k}"
             pattern: str = r"(?<![a-zA-Z])" + k + r"(?![a-zA-Z])"
             string = sub(pattern, repl, string)
         return string
@@ -456,7 +459,7 @@ class Element:
         return sympify(self._str_expr(substitute=substitute))
 
     def to_latex(self) -> str:
-        return latex(self.to_sympy(substitute=False))
+        return f"Z = {latex(self.to_sympy(substitute=False))}"
 
 
 class Connection:
@@ -527,16 +530,41 @@ class Connection:
     def to_string(self, decimals: int = -1) -> str:
         raise Exception("Not yet implemented!")
 
-    def get_elements(
-        self, flattened: bool = True
-    ) -> List[Union[Element, "Connection"]]:
+    def get_connections(self, flattened: bool = True) -> List["Connection"]:
         """
-        Get a list of elements and connections nested inside this connection.
+        Get the connections in this circuit.
 
         Parameters
         ----------
         flattened: bool = True
-            Whether the returned list should only contain elements or a combination of elements and connections.
+            Whether or not the connections should be returned as a list of all connections or as a list connections that may also contain more connections.
+
+        Returns
+        -------
+        List[Connection]
+        """
+        if flattened:
+            connections: List["Connection"] = []
+            for element in self._elements:
+                if isinstance(element, Connection):
+                    connections.extend(
+                        reversed(element.get_connections(flattened=flattened))
+                    )
+            return list(reversed(connections))
+        return list(
+            reversed(filter(lambda _: isinstance(_, Connection), self._elements))
+        )
+
+    def get_elements(
+        self, flattened: bool = True
+    ) -> List[Union[Element, "Connection"]]:
+        """
+        Get the elements in this circuit.
+
+        Parameters
+        ----------
+        flattened: bool = True
+            Whether or not the elements should be returned as a list of only elements or as a list of connections containing elements.
 
         Returns
         -------
@@ -552,6 +580,21 @@ class Connection:
             return list(reversed(elements))
         return list(reversed(self._elements))
 
+    def substitute_element(self, ident: int, element: Element) -> bool:
+        elem_con: Union[Element, Connection]
+        for elem_con in self.get_elements(flattened=False):
+            if isinstance(elem_con, Element):
+                if elem_con.get_identifier() == ident:
+                    element._assign_identifier(ident)
+                    index: int = self._elements.index(elem_con)
+                    self._elements.pop(index)
+                    self._elements.insert(index, element)
+                    return True
+            else:
+                if elem_con.substitute_element(ident, element) is True:
+                    return True
+        return False
+
     def impedance(self, f: float) -> complex:
         """
         Calculates the impedance of the connection at a single frequency.
@@ -660,4 +703,4 @@ class Connection:
         return sympify(self._str_expr(substitute=substitute))
 
     def to_latex(self) -> str:
-        return latex(self.to_sympy(substitute=False))
+        return f"Z = {latex(self.to_sympy(substitute=False))}"
