@@ -18,4 +18,15 @@
 # the LICENSES folder.
 
 from .data_set import DataSet
-from .formats import parse_data, parse_csv, parse_ids, parse_spreadsheet
+from .formats import (
+    UnsupportedFileFormat,
+    get_parsers,
+    parse_csv,
+    parse_data,
+    parse_dfr,
+    parse_i2b,
+    parse_ids,
+    parse_mpt,
+    parse_p00,
+    parse_spreadsheet,
+)
